searchData={"items":[{"type":"module","title":"TextParser","doc":"","ref":"TextParser.html"},{"type":"macro","title":"TextParser.__using__/1","doc":"When used, defines a custom parser module.","ref":"TextParser.html#__using__/1"},{"type":"macro","title":"Example - TextParser.__using__/1","doc":"defmodule MyParser do\n      use TextParser\n\n      def validate(%TextParser.Tokens.Tag{value: value} = tag) do\n        if String.length(value) >= 66, do: {:error, \"tag too long\"}, else: {:ok, tag}\n      end\n    end","ref":"TextParser.html#__using__/1-example"},{"type":"function","title":"TextParser.get/2","doc":"Returns tokens of the specified type from a Text struct.","ref":"TextParser.html#get/2"},{"type":"function","title":"Examples - TextParser.get/2","doc":"iex> text = TextParser.parse(\"Check out https://example.com #elixir\")\n    iex> TextParser.get(text, URL)\n    [%TextParser.Tokens.URL{value: \"https://example.com\", position: {10, 29}}]","ref":"TextParser.html#get/2-examples"},{"type":"function","title":"TextParser.parse/1","doc":"","ref":"TextParser.html#parse/1"},{"type":"function","title":"TextParser.parse/2","doc":"Parses the given text and returns a Text struct with extracted tokens.","ref":"TextParser.html#parse/2"},{"type":"function","title":"Options - TextParser.parse/2","doc":"* `:extract` - A list of token extractor modules that implement the `TextParser.Token` behaviour.","ref":"TextParser.html#parse/2-options"},{"type":"function","title":"Examples - TextParser.parse/2","doc":"# Extract URLs only\n    iex> TextParser.parse(\"Check out https://example.com\", extract: [URL])\n    %TextParser.Text{\n      value: \"Check out https://example.com\",\n      tokens: [\n        %TextParser.Tokens.URL{\n          value: \"https://example.com\",\n          position: {10, 29}\n        }\n      ]\n    }\n\n    # Extract URLs and tags\n    iex> TextParser.parse(\"Check out https://example.com #elixir\", extract: [URL, Tag])\n    %TextParser.Text{\n      value: \"Check out https://example.com #elixir\",\n      tokens: [\n        %TextParser.Tokens.URL{\n          value: \"https://example.com\",\n          position: {10, 29}\n        },\n        %TextParser.Tokens.Tag{\n          value: \"#elixir\",\n          position: {30, 37}\n        }\n      ]\n    }","ref":"TextParser.html#parse/2-examples"},{"type":"behaviour","title":"TextParser.Behaviour","doc":"Behaviour for implementing custom text parsers.","ref":"TextParser.Behaviour.html"},{"type":"behaviour","title":"Example - TextParser.Behaviour","doc":"defmodule MyParser do\n      use TextParser\n\n      def validate(%TextParser.Tokens.Tag{value: value} = tag) do\n        if String.length(value) >= 66, do: {:error, \"tag too long\"}, else: {:ok, tag}\n      end\n    end","ref":"TextParser.Behaviour.html#module-example"},{"type":"callback","title":"TextParser.Behaviour.validate/1","doc":"Callback for validating a token.\n\nThis should be implemented by each parser module to define its specific\nvalidation rules. The callback receives a token struct and should return\neither `{:ok, token}` or `{:error, reason}`.","ref":"TextParser.Behaviour.html#c:validate/1"},{"type":"module","title":"TextParser.Text","doc":"","ref":"TextParser.Text.html"},{"type":"function","title":"TextParser.Text.new/1","doc":"Creates a new Text struct from the given text value.","ref":"TextParser.Text.html#new/1"},{"type":"type","title":"TextParser.Text.t/0","doc":"","ref":"TextParser.Text.html#t:t/0"},{"type":"behaviour","title":"TextParser.Token","doc":"Behaviour for implementing custom token extractors.","ref":"TextParser.Token.html"},{"type":"behaviour","title":"Example - TextParser.Token","doc":"defmodule MyApp.CustomToken do\n      use TextParser.Token,\n        pattern: ~r/(?:^| )(@S*)/u,\n        trim_chars: [\",\", \".\", \"!\", \"?\"]\n\n      def is_valid?(token) do\n        # implement validation logic\n      end\n    end","ref":"TextParser.Token.html#module-example"},{"type":"macro","title":"TextParser.Token.__using__/1","doc":"When used, defines the token struct and implements the behaviour.","ref":"TextParser.Token.html#__using__/1"},{"type":"macro","title":"Options - TextParser.Token.__using__/1","doc":"* `:pattern` - Required. A regex pattern to match tokens in text\n  * `:trim_chars` - Optional. A list of characters to trim from the end of matched tokens","ref":"TextParser.Token.html#__using__/1-options"},{"type":"callback","title":"TextParser.Token.extract/1","doc":"Callback for extracting tokens from text.\n\nThe implementation should return a list of token structs, where each token\nhas a `:value` and `:position` field. The position should be a tuple of\n`{start_pos, end_pos}` indicating the token's position in the text.","ref":"TextParser.Token.html#c:extract/1"},{"type":"callback","title":"TextParser.Token.is_valid?/1","doc":"Callback for validating a token value.\n\nThis should be implemented by each token module to define its specific\nvalidation rules.","ref":"TextParser.Token.html#c:is_valid?/1"},{"type":"module","title":"TextParser.Tokens.Mention","doc":"","ref":"TextParser.Tokens.Mention.html"},{"type":"type","title":"TextParser.Tokens.Mention.t/0","doc":"","ref":"TextParser.Tokens.Mention.html#t:t/0"},{"type":"module","title":"TextParser.Tokens.Tag","doc":"","ref":"TextParser.Tokens.Tag.html"},{"type":"type","title":"TextParser.Tokens.Tag.t/0","doc":"","ref":"TextParser.Tokens.Tag.html#t:t/0"},{"type":"module","title":"TextParser.Tokens.URL","doc":"","ref":"TextParser.Tokens.URL.html"},{"type":"type","title":"TextParser.Tokens.URL.t/0","doc":"","ref":"TextParser.Tokens.URL.html#t:t/0"},{"type":"extras","title":"TextParser","doc":"# TextParser\n\n[![CI](https://github.com/solnic/text_parser/actions/workflows/ci.yml/badge.svg)](https://github.com/solnic/text_parser/actions/workflows/ci.yml) [![Hex pm](https://img.shields.io/hexpm/v/text_parser.svg?style=flat)](https://hex.pm/packages/text_parser) [![hex.pm downloads](https://img.shields.io/hexpm/dt/text_parser.svg?style=flat)](https://hex.pm/packages/text_parser)\n\nTextParser is an Elixir library for extracting and validating structured tokens from text, such as URLs, hashtags, and mentions. It provides built-in token types and allows you to define custom parsers with specific validation rules.\n\nThis library was extracted from [justcrosspost.app](https://justcrosspost.app) where processing tags, mentions and URLs for Bluesky is [kinda tricky](https://docs.bsky.app/docs/advanced-guides/post-richtext).","ref":"readme.html"},{"type":"extras","title":"Installation - TextParser","doc":"Add `text_parser` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:text_parser, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - TextParser","doc":"#","ref":"readme.html#usage"},{"type":"extras","title":"Basic Parsing - TextParser","doc":"By default, TextParser extracts URLs, hashtags, and mentions:\n\n```elixir\ntext = \"Check out https://elixir-lang.org #elixir @joe\"\n\nresult = TextParser.parse(text)\n\n# Get URLs\nurls = TextParser.get(result, TextParser.Tokens.URL)\n# => [%TextParser.Tokens.URL{value: \"https://elixir-lang.org\", position: {10, 32}}]\n\n# Get hashtags\ntags = TextParser.get(result, TextParser.Tokens.Tag)\n# => [%TextParser.Tokens.Tag{value: \"#elixir\", position: {33, 40}}]\n\n# Get mentions\nmentions = TextParser.get(result, TextParser.Tokens.Mention)\n# => [%TextParser.Tokens.Mention{value: \"@joe\", position: {41, 45}}]\n```\n\n#","ref":"readme.html#basic-parsing"},{"type":"extras","title":"Selective Token Extraction - TextParser","doc":"You can specify which token types to extract:\n\n```elixir\nalias TextParser.Tokens.{URL, Tag}\n\n# Extract only URLs and hashtags\nresult = TextParser.parse(\"Check out https://elixir-lang.org #elixir @joe\", extract: [URL, Tag])\n\nTextParser.get(result, URL) # => [%URL{...}]\nTextParser.get(result, Tag) # => [%Tag{...}]\n```\n\n#","ref":"readme.html#selective-token-extraction"},{"type":"extras","title":"Custom Tokens - TextParser","doc":"You can create custom token types to extract different patterns from text. Here's an example of a token that extracts ISO 8601 dates:\n\n```elixir\ndefmodule MyParser.Tokens.Date do\n  use TextParser.Token,\n    # Match YYYY-MM-DD format, requiring space or start of string before the date\n    pattern: ~r/(?:^|\\s)(\\d{4}-(?:0[1-9]|1[0-2])-(?:0[1-9]|[12]\\d|3[01]))/,\n    trim_chars: [\",\", \".\", \"!\", \"?\"]\n\n  @impl true\n  def is_valid?(date_text) when is_binary(date_text) do\n    case Date.from_iso8601(date_text) do\n      {:ok, _date} -> true\n      _ -> false\n    end\n  end\n\n  def is_valid?(_), do: false\nend\n\n# Usage\ntext = \"Meeting on 2024-01-15, conference on 2024-02-30, party on 2024-12-31!\"\nresult = TextParser.parse(text, extract: [MyParser.Tokens.Date])\n\ndates = TextParser.get(result, MyParser.Tokens.Date)\n# => [\n#   %MyParser.Tokens.Date{value: \"2024-01-15\", position: {11, 21}},\n#   %MyParser.Tokens.Date{value: \"2024-12-31\", position: {47, 57}}\n# ]\n# Note: 2024-02-30 is filtered out as it's not a valid date\n```\n\nCustom tokens require:\n1. A regex `pattern` that captures the token in the first capture group\n2. Optional `trim_chars` to remove trailing punctuation\n3. An `is_valid?/1` function that validates the extracted value\n\nYou can mix custom tokens with built-in ones:\n\n```elixir\nalias TextParser.Tokens.{URL, Tag}\nalias MyParser.Tokens.Date\n\nresult = TextParser.parse(\n  \"Meeting on 2024-01-15 at https://example.com #elixir\",\n  extract: [URL, Tag, Date]\n)\n\nTextParser.get(result, Date)  # => [%Date{value: \"2024-01-15\", position: {11, 21}}]\nTextParser.get(result, URL)   # => [%URL{value: \"https://example.com\", position: {25, 44}}]\nTextParser.get(result, Tag)   # => [%Tag{value: \"#elixir\", position: {45, 52}}]\n```\n\n#","ref":"readme.html#custom-tokens"},{"type":"extras","title":"Custom Parsers - TextParser","doc":"You can create custom parsers with specific validation rules:\n\n```elixir\ndefmodule BlueskyParser do\n  use TextParser\n\n  alias TextParser.Tokens.Tag\n\n  # Enforce Bluesky's 64-character limit for hashtags\n  def validate(%Tag{value: value} = tag) do\n    if String.length(value) >= 66,\n      do: {:error, \"tag too long\"},\n      else: {:ok, tag}\n  end\n\n  # Allow other token types to pass through\n  def validate(token), do: {:ok, token}\nend\n\n# Usage\ntext = \"Check out #elixir and #this_is_a_very_long_hashtag_that_exceeds_bluesky_limit\"\n\nresult = BlueskyParser.parse(text)\n\n# Only valid tags are included\nTextParser.get(result, Tag)\n# => [%Tag{value: \"#elixir\", position: {10, 17}}]\n```\n\n#","ref":"readme.html#custom-parsers"},{"type":"extras","title":"Token Information - TextParser","doc":"Each token includes its value and position in the original text:\n\n```elixir\ntext = \"Check https://elixir-lang.org #elixir\"\nresult = TextParser.parse(text)\n\n[url] = TextParser.get(result, TextParser.Tokens.URL)\n\nurl.value     # => \"https://elixir-lang.org\"\nurl.position  # => {6, 29} (start and end byte positions)\n```\n\n:warning: The `position` tuple uses a range format where:\n\n- The start position is inclusive (the first byte of the token)\n- The end position is exclusive (one byte past the last byte of the token)\n\nFor example, with `position: {6, 29}`, the token starts at byte 6 and ends at byte 28. This format makes it easy to:\n- Calculate token length: `end - start` (29 - 6 = 23 bytes)\n- Extract the token from text: `binary_part(text, start, end - start)`","ref":"readme.html#token-information"},{"type":"extras","title":"Documentation - TextParser","doc":"The full documentation can be found at [https://hexdocs.pm/text_parser](https://hexdocs.pm/text_parser).","ref":"readme.html#documentation"},{"type":"extras","title":"0.1.0 2025-03-13","doc":"# 0.1.0 2025-03-13\n\nFirst release! 🎉\n\n- `TextParser.parse/2` for parsing text into tokens\n- `TextParser.get/2` for extracting tokens by type\n- `TextParser.Token` behaviour for defining custom token types\n- `TextParser` behaviour for defining custom parsers","ref":"changelog.html"}],"content_type":"text/markdown"}